@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------*/
-/*  Copyright (C) 2004-2007  Serge Iovleff
+/*  Copyright (C) 2004-20014  Serge Iovleff
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as
@@ -42,184 +42,11 @@
 #include "../include/STK_Funct_poisson_raw.h"
 #include "../include/STK_Funct_raw.h"
 
-#include "../../Arrays/include/STK_Array2DVector.h"
-
-namespace STK
-{
-
-namespace Funct
-{
-
-/** @ingroup Analysis
- *  @brief Compute the continued fraction of the beta function.
- *
- *  Compute the continued fraction:
- * \f[
- *    \frac{a_1}{b_1+}\frac{a_2}{b_2+}\frac{a_3}{b_3+}\ldots
- * \f]
- * with \f$ a_1 = 1 \f$, \f$ a_{n+1} = c_n d_n /((a+2n-2)(a+2n))\f$
- * and \f$ b_{n+1} = (1-(a+b)x) + c_n + d_{n+1}\f$, where
- * \f[
- *   c_n = \frac{n(b-n)x}{a+2n-1} \qquad
- *   d_n = \frac{(a+n-1)(a+b+n-1)x}{a+2n-1}.
- * \f]
- *  @param a first parameter, must be >0
- *  @param b second parameter, must be >0
- *  @param x value to evaluate the function
- *  @param lower_tail @c true if we want the lower tail, @c false otherwise
- *  @param iterMax maximal number of iteration
- *  @return the value of the beta ratio function using the continued fraction
- *  method
- */
-static Real betaRatio_cf( Real const& a
-                        , Real const& b
-                        , Real const& x
-                        , bool lower_tail = true
-                        , int const& iterMax = 1000
-                        )
-{
-  // Constants
-  Real sum = a+b, y = (0.5 -x) + 0.5;
-  /* Compute the Function:
-   *  B(a,b,x,y) = \frac{\Gamma(a+b)}{\Gamma(a) \Gamma(b)} x^{a} (1-x)^{b}
-   */
-  Real bt = ( Const::_1_SQRT2PI_*std::sqrt((a*b)/sum)
-            * std::exp( ( gammaLnStirlingError(sum)
-                -gammaLnStirlingError(a)
-                -gammaLnStirlingError(b)
-                )
-              -(dev0(a, sum*x) + dev0(b, y*sum))
-              )
-        );
-  if (bt == 0.) return lower_tail ? 0. : 1.;
-
-  // initialize numerator
-  Real Nold = 0, Ncur = 1.; // = a_1 = 1
-  // initialize denominator
-  Real Dold = 1, Dcur = (1.+a*y-b*x)/(a+1.); // = b_1
-
-  // normalize if necessary
-  if (std::abs(Dcur) > 4294967296.0)
-  {
-    Ncur /= Dcur;
-    Dold /= Dcur;
-    Dcur = 1.;
-  }
-
-  // auxiliary constant variables
-  const Real x2_16 = (x*x)/16., x_4 = x/4.
-           , cte_b1 = y/2 + 0.5
-           , cte_b2 = (a-1.)*(2.*b+a-1.)*x/2.
-           , cte_a1 = a*(a-2.)*(2.*b+a)*(2.*b+a-2.)*x2_16
-           , cte_a2 = cte_b2/2.
-           ;
-  // result
-  Real cf = 1/Dcur;
-  // auxiliary variables
-  Real ap2n = a+2., ap2nm1 = a+1.;
-  // iterations
-  for (int n=1; /*n<=iterMax*/; n++, ap2n += 2, ap2nm1 += 2)
-  {
-    // compute a_n
-    Real a_n = - (x_4 - cte_a2/ap2nm1) * (x_4 + cte_a2/ap2nm1) - cte_a1/(ap2n*(ap2n-2)) ;
-
-    // check trivial case
-    if (a_n*Arithmetic<Real>::epsilon() == 0.) break;
-    // compute b_n
-    Real b_n = cte_b1 - cte_b2/(ap2nm1*(ap2n+1));
-    // compute numerator and denominator
-    Real anew = b_n * Ncur + a_n * Nold;
-    Nold = Ncur;
-    Ncur = anew;
-    // check for overflow
-    if (Ncur*Arithmetic<Real>::epsilon() == 0.) { cf =0.; break;}
-    anew = b_n * Dcur + a_n * Dold;
-    Dold = Dcur;
-    Dcur = anew;
-    // normalize if necessary
-    if (std::abs(Dcur) > 4294967296.0)
-    {
-      Ncur /= Dcur;
-      Nold /= Dcur;
-      Dold /= Dcur;
-      Dcur = 1.;
-    }
-    // normalize if necessary with 2^32
-    if (std::abs(Ncur) > 4294967296.0)
-    {
-      Ncur /= 4294967296.0;
-      Nold /= 4294967296.0;
-      Dold /= 4294967296.0;
-      Dcur /= 4294967296.0;
-    }
-    // test D_n not too small
-    if (std::abs(Dcur) != 0)
-    {
-      Real cfold = cf;
-      cf = Ncur/Dcur;
-      // check cv
-      if (std::abs(cf - cfold) < std::abs(cf)*Arithmetic<Real>::epsilon())
-      { break;}
-    }
-  }
-#ifdef STK_DEBUG
-  // result
-  Real res = lower_tail ? bt*cf/a : 1-bt*cf/a;
-  if (!Arithmetic<Real>::isFinite(res))
-  {
-    stk_cout << _T("in cf : bt= ") << bt
-              << _T(" cf = ") << cf
-              << _T(" a  = ") << a
-              << _T(" b  = ") << b
-              << _T(" x  = ") << x
-              << _T("lt =  ") << lower_tail
-              << _T("\n");
-  }
-  return res;
-#else
-  return lower_tail ? bt*cf/a : 1-bt*cf/a;
-#endif
-
-}
-
-/* @ingroup Analysis
- *  @brief compute the coefs of the beta Ratio function in
- *  its asymptotic expansion.
- *
- *  Given the n-1 first coefs, compute and return the n-th
- *  coefficient. The value is stored back to the Vector. The
- *  formula for computing the coefficient is
- *  \[
- *     a_{n}
- *      = \frac{b-1}{(2n+1)!}+\frac{1}{n}
- *        \sum_{k=1}^{n-1} \left[
- *                          \frac{kb-n}{(2k+1)!} p_{n-k}
- *                         \right]
- *  \f]
- *  @param A vector of dimension 1:(n-1), n>=2, of the coefs
- *  @param b second parameter of the beta function
- */
-//static Real coefs_ae(Real const& b, Vector &A)
-//{
-//  // get the number of exising coefs
-//  int n = A.size();
-//  // initialize the sum
-//  Real sum = 0.0;
-//  // update sum
-//  for (int k=1; k<n; k++)
-//  {
-//    sum += (k*b/n-1)*A[n-k]/(factorial(2*k+1));
-//  }
-//  // compute result
-//  sum += (b-1)/factorial(2*n+1);
-//  // save it in A
-//  A.push_back(sum);
-//  // and return it
-//  return sum;
-//}
+#include "Arrays/include/STK_Array2DVector.h"
+#include "STKernel/include/STK_Real.h"
 
 #define d1(z) (0.5)
-#define d2(z) (z/8. - 1./20.)
+#define d2(z) (z/8. - 0.05)
 #define d3(z) (z*(z/48.-1./40.)+1./105.)
 #define d4(z) (z*(z*(z/384.-1./160.)+101./16800.)-3./1400.)
 #define d5(z) (z*(z*(z*(z/3840.-1./960.)+61./33600.)-13./8400.) \
@@ -233,6 +60,26 @@ static Real betaRatio_cf( Real const& a
               +52037./1681680000.)-10851./1191190000.)
 
 
+namespace STK
+{
+
+namespace Funct
+{
+
+/** @ingroup Analysis
+ *  @brief Compute the function:
+ *  B(a,b,x,y) = \frac{ x^{a} (1-x)^{b}}{B(a,b)}
+ *  using  (a+b) * (p*log(x/p)+q*log(y/q))
+ */
+static Real dbinom(Real const& a, Real const& b, Real const& x, bool xm1)
+{
+  Real s  = a+b, sx = s*x, sy = s-sx;
+  return ( Const::_1_SQRT2PI_*std::sqrt((a*b)/s)
+         * std::exp( ( gammaLnStirlingError(s)-gammaLnStirlingError(a)-gammaLnStirlingError(b))
+                    -(xm1 ? dev0(a, sy)+dev0(b, sx) : dev0(a, sx)+dev0(b, sy))
+                   )
+        );
+}
 /** @ingroup Analysis
  *  @brief Compute the incomplete beta function ratio I_x(a,b)
  *  using its asymptotic expansion.
@@ -242,17 +89,17 @@ static Real betaRatio_cf( Real const& a
  *  @param a first parameter, must be >0
  *  @param b second parameter, must be >0
  *  @param x value to evaluate the function
- *  @param xm1 true if we want to evaluate the function at 1-x
+ *  @param xm1 @c true if we want to evaluate the function at 1-x, @c false otherwise
  *  @param lower_tail @c true if we want the lower tail, @c false otherwise
  *  @return the value of the beta ratio function using its asymptotic expansion
  **/
-static inline Real betaRatio_ae( Real const& a
-                               , Real const& b
-                               , Real const& x
-                               , bool xm1
-                               , bool lower_tail
-                               )
+Real betaRatio_ae( Real const& a, Real const& b, Real const& x
+                 , bool xm1, bool lower_tail
+                 )
 {
+#ifdef STK_BETARATIO_DEBUG
+  stk_cout << _T("BetaRatio_ae(") << a << _T(", ") << b << _T(", ") << x << _T(")\n");
+#endif
   // Compute b-1
   Real bm1 = b-1;
   // compute \nu = a+(b-1)/2
@@ -336,32 +183,101 @@ static inline Real betaRatio_ae( Real const& a
                     - (num / den) * poisson_pdf_raw(b, D);
 }
 
+/** @ingroup Analysis
+ *  @brief Compute the incomplete beta function ratio I_x(a,b)
+ *  using its series representation.
+ *
+ * Compute the incomplete beta function ratio I_x(a,b)
+ * using it's series representation.
+ * \f[
+ * \frac{x^a}{B(a,b)}
+ * \left(  \frac{1}{a}
+ *       + \sum_{n=1}^{\infty} \frac{(1-b)(2-b)\ldots(n-b)}{n! (a+n)} x^n
+ * \right)
+ * \f]
+ *  @param a first parameter, must be >0
+ *  @param b second parameter, must be >0
+ *  @param x value to evaluate the function
+ *  @param lower_tail @c true if we want the lower tail, @c false otherwise
+ *  @return the value of the beta ratio function using its series representation
+ **/
+static Real betaRatio_sr( Real const& a, Real const& b, Real x
+                        , bool xm1, bool lower_tail
+                        )
+{
+#ifdef STK_BETARATIO_DEBUG
+  stk_cout << _T("BetaRatio_sr(") << a << _T(", ") << b << _T(", ") << x << _T(")\n");
+#endif
+  // constants
+  Real s = a+b, sx = s*x, sy = s-sx;
+  // compute B(a,b,x,y) = \frac{\Gamma(a+b)}{\Gamma(a) \Gamma(b)} x^{a}
+  Real bt = ( Const::_1_SQRT2PI_*sqrt((a*b)/s)
+            * exp( ( gammaLnStirlingError(s)
+                    -(gammaLnStirlingError(a)+gammaLnStirlingError(b))
+                   )
+                 + (xm1 ? sy- (dev0(a, sy) + dev0(b, s)) : sx- (dev0(a, sx) + dev0(b, s)))
+                 )
+            );
+  Real y = (0.5 - x) + 0.5;
+  if (xm1) std::swap(x,y);
+  // check factor
+  if (!bt) return lower_tail ? 0. : 1.;
+  // parameters
+  int n = 1;
+  Real sum = 0.0, c= 1., term;
+  do
+  {
+    sum += (term =  (c *= (1.-b/n)*x) / (a+n));
+    ++n;
+  }
+  while (std::abs(term) > std::abs(sum) * Arithmetic<Real>::epsilon());
+  // return result
+  return lower_tail ? bt*(1./a + sum) : (1.-bt/a-bt*sum);
+}
+
 /** @ingroup Analysis
  *  @brief Compute the recurrence formula of the incomplete beta
  *  ratio function.
  *
- *  Compute the recurrence formula of the incomplete beta
- *  ratio function I_x(a,b).
+ *  Compute the recurrence formula of the incomplete beta ratio
+ *  function I_x(a,b):
+ *  \[
+ *  x^a (1-x)^b \sum_{j=1}^n \frac{\Gamma(a+b+j-1)}{\Gamma(b) \Gamma(a+j)} x^n
+ *  \]
  *
- *  @param s a+b
- *  @param a first parameter
+ *  @param a,b parameters
  *  @param x value to evaluate the function
+ *  @param xm1 @c true if we want 1-x rather than x, @c false otherwise
  *  @param n number of step
  **/
-static inline Real serie_up( Real const& s
-                           , Real const& a
-                           , Real const& x
-                           , int const& n
-                           )
+static Real serie_up( Real const& a, Real const& b, Real x, bool xm1, int n)
 {
-  // initilize d0 and sum
-  Real sum = 1./a;
-  Real di  = 1./a;
-  // compute \sum_{i=0}^{n-1} \frac{d_i x^i}{a}
-  for (int i=1; i<n; i++)
-    sum += (di *= x*(s-1+i)/(a+i));
+  // trivial case
+  if (n==1) return dbinom(a, b, x, xm1)/a;
+
+  // constants
+  Real s = a+b, sx=s*x, sy = s- sx;
+  Real x0 = (xm1) ?  (0.5 -x) + 0.5 : x;
+  int l = std::min(std::max(0, (int)round( xm1 ? (sy-a+1)/x : (sx-a+1)/x0)), n-1);
+  Real sum1 = 0., sum2 = 1., s0 = s+l, a0 = a+l;
+  // first serie
+  for (int k=n-l-2; k>=0; --k)  { sum2 = 1. + sum2 *x0*(s0+k)/(a0+k+1);}
+  // second serie
+  if (l>0)
+  {
+    sum1 = 1.;
+    for (int k=1; k<l; ++k) { sum1 = 1.+ sum1*(a+k)/((s+k-1)*x0);}
+    sum1 /= ((s0-1)*x0);
+  }
   // return result
-  return sum;
+  return dbinom(a+l, b, x, xm1) *(sum2/a0 + sum1);
+
+//  stk_cout << _T("bt =") << dbinom(a, b, x, xm1) << _T(" serie =") << sum/a << " res =" << dbinom(a, b, x, xm1)*sum/a<< "\n";
+//  stk_cout << _T("n =") << n << _T(", l =") << l << "\n";
+//  Real sum = 1.;
+//  for (int k=n-2; k>=0; --k)  { sum = 1. + sum *x0*(s+k)/(a+k+1);}
+//  stk_cout << _T("bt_l =") << dbinom(a+l, b, x, xm1) << _T(" serie_l =") << (sum2/a0 + sum1) << " res=" << dbinom(a+l, b, x, xm1) *(sum2/a0 + sum1)<< "\n";
+//  return dbinom(a, b, x, xm1) *sum/a;
 }
 
 /** @ingroup Analysis
@@ -377,13 +293,13 @@ static inline Real serie_up( Real const& s
  *  @param xm1 true if we want to evaluate the function at 1-x
  *  @param lower_tail @c true if we want the lower tail, @c false otherwise
  **/
-static inline Real betaRatio_up( Real const& a
-                               , Real const& b
-                               , Real const& x
-                               , bool xm1
-                               , bool lower_tail
-                               )
+static Real betaRatio_up( Real const& a, Real const& b, Real const& x
+                        , bool xm1, bool lower_tail
+                        )
 {
+#ifdef STK_BETARATIO_DEBUG
+  stk_cout << _T("BetaRatio_up(") << a << _T(", ") << b << _T(", ") << x << _T(")\n");
+#endif
   // number of iterations
   int n = int(a);
   // compute residual
@@ -391,125 +307,141 @@ static inline Real betaRatio_up( Real const& a
   // zero case
   if (!a0) { --n; a0=1.;}
   // sum
-  Real s= a0+b;
-  // compute x and 1-x
-  Real x0, y;
-  if (xm1)
-  { x0 = (0.5 -x) + 0.5; y =x;}
-  else
-  { x0 = x; y = (0.5 -x) + 0.5;}
-  /* Compute the constant
-   *  B(a,b,x,y) = \frac{\Gamma(a+b)}{\Gamma(a) \Gamma(b)} x^a (1-x)^b
-   * using saddle-point expansion
-   */
-  Real bt = ( Const::_1_SQRT2PI_*sqrt((a0*b)/s)
-            * exp( ( gammaLnStirlingError(s)
-                    -gammaLnStirlingError(a0)
-                    -gammaLnStirlingError(b)
-                   )
-                   -(dev0(a0, s*x0) + dev0(b, y*s))
-                 )
-            );
-  // check trivial case
-  if (!bt)
-    return betaRatio_ae(b, a0, x, !xm1, !lower_tail);
-  // compute serie
-    bt *= serie_up(s, a0, x0, n);
-  // general case
-  return  lower_tail ? betaRatio_ae(b, a0, x, !xm1, !lower_tail) - bt
-                     : betaRatio_ae(b, a0, x, !xm1, !lower_tail) + bt;
+  Real c=  serie_up(a0, b, x, xm1, n) * (lower_tail ? - 1. : 1.);
+  return (b<=15) ? betaRatio_sr(a0, b, x, xm1, lower_tail) +c
+                 : betaRatio_ae(b, a0, x, !xm1, !lower_tail) +c;
 }
 
 /** @ingroup Analysis
- *  @brief Compute the incomplete beta function ratio I_x(a,b)
- *  using its series representation.
- *
- * Compute the incomplete beta function ratio I_x(a,b)
- * using it's serie representation.
+ *  Compute the continued fraction:
  * \f[
- * \frac{x^a}{B(a,b)}
- * \left(  \frac{1}{a}
- *       + \sum_{n=1}^{\infty} \frac{(1-b)(2-b)\ldots(n-b)}{n! (a+n)}
- * \right)
+ *    \frac{a_1}{b_1+}\frac{a_2}{b_2+}\frac{a_3}{b_3+}\ldots
  * \f]
+ * with \f$ a_1 = 1 \f$, \f$ a_{n+1} = c_n d_n /((a+2n-2)(a+2n))\f$
+ * and \f$ b_{n+1} = (1-(a+b)x) + c_n + d_{n+1}\f$, where
+ * \f[
+ *   c_n = \frac{n(b-n)x}{a+2n-1} \qquad
+ *   d_n = \frac{(a+n-1)(a+b+n-1)x}{a+2n-1}.
+ * \f]
+ * We are using a new formulation of the
  *  @param a first parameter, must be >0
  *  @param b second parameter, must be >0
  *  @param x value to evaluate the function
+ *  @param xm1 @c true if we are looking for 1-x rather than x, @c false otherwise
  *  @param lower_tail @c true if we want the lower tail, @c false otherwise
- *  @return the value of the beta ratio function using its series representation
- **/
-static Real betaRatio_sr( Real const& a
-                        , Real const& b
-                        , Real const& x
-                        , bool lower_tail
-                        )
-{
-  // Constant
-  Real sum = a+b;
-  // compute B(a,b,x,y) = \frac{\Gamma(a+b)}{\Gamma(a) \Gamma(b)} x^{a}
-  Real bt = ( Const::_1_SQRT2PI_*sqrt((a*b)/sum)
-            * exp( sum*x
-                 + ( gammaLnStirlingError(sum)
-                    -(gammaLnStirlingError(a)+gammaLnStirlingError(b))
-                   )
-                 -(dev0(a, sum*x) + dev0(b, sum))
+ *  @return the value of the beta ratio function using the continued fraction
+ *  method
+ */
+Real betaRatio_cf( Real const& a, Real const& b, Real x
+                 , bool xm1, bool lower_tail
                  )
-            );
-  // check factor
-  if (!bt) return lower_tail ? 0. : 1.;
+{
+#ifdef STK_BETARATIO_DEBUG
+  stk_cout << _T("BetaRatio_cf(") << a << _T(", ") << b << _T(", ") << x << _T(")\n");
+#endif
   // parameters
-  Real term, n= 1., c = 1.;
-  // sum = \sum_{n=1}^\infty (1-b/n) x^n /(a+n)
-  sum = 0.0;
-  do
+  Real s  = a+b, sx = s*x, sy = s-sx;
+  /* Compute the function:
+   *  B(a,b,x,y) = \frac{ x^{a} (1-x)^{b}}{B(a,b)}
+   *  using  (a+b) * (p*log(x/p)+q*log(y/q))
+   */
+  Real bt = dbinom(a, b, x, xm1);
+  if (bt*Arithmetic<Real>::epsilon() == 0.) return lower_tail ? 0. : 1.;
+
+  // initialize numerator
+  Real Nold = 0, Ncur = 1.; // = a_1 = 1
+  // initialize denominator
+  Real Dold = 1., Dcur = xm1 ? 1.-sy/(a+1.) : 1.-sx/(a+1.); // = b_1
+  // normalize if necessary
+  if (std::abs(Dcur) > 4294967296.)
   {
-    sum += (term =  (c *= (1.-b/n)*x) / (a+n));
-    ++n;
+    Ncur /= Dcur;
+    Dold /= Dcur;
+    Dcur = 1.;
   }
-  while (std::abs(term) > std::abs(sum) * Arithmetic<Real>::epsilon());
-  // return result
-  return lower_tail ? bt*(1/a + sum)
-                    : (1-bt/a)-(bt*sum);
+  // auxiliary constant variables
+  const Real    x_4 = xm1 ? (0.125 - x/4.) + 0.125 : x/4.;
+  const Real cte_b1 = xm1 ? x/2. + 0.5 : (0.5 - x/2.) + 0.5;
+  const Real cte_b2 = xm1 ? (a-1.)*(b+s-1.)*((0.25-x/2.)+0.25) : (a-1.)*(b+s-1.)*x/2.;
+  const Real cte_a1 = a*(a-2.)*(b+s)*(b+s-2.), cte_a2 = (a-1.)*(b+s-1.)*x_4;
+  // result
+  Real cf = 1/Dcur;
+  // auxiliary variables
+  Real ap2n = a+2., ap2nm1 = a+1.,ap2nm2 = a;
+  // iterations
+  for (int n=1; /*n<=iterMax*/; n++, ap2n += 2., ap2nm1 += 2., ap2nm2 += 2.)
+  {
+    // compute a_n
+    Real a_n = (cte_a2/ap2nm1 - x_4) * (x_4 + cte_a2/ap2nm1) - cte_a1*(x_4/ap2n)*(x_4/ap2nm2) ;
+    // check trivial case
+    if (a_n*Arithmetic<Real>::epsilon() == 0.) break;
+    // compute b_n
+    Real b_n = cte_b1 - cte_b2/(ap2nm1*(ap2n+1));
+    // compute numerator and denominator
+    Real anew = b_n * Ncur + a_n * Nold;
+    Nold = Ncur;
+    Ncur = anew;
+    // check for underflow
+    if (Ncur*Arithmetic<Real>::epsilon() == 0.) { cf =0.; break;}
+    anew = b_n * Dcur + a_n * Dold;
+    Dold = Dcur;
+    Dcur = anew;
+    // normalize if necessary
+    if (std::abs(Dcur) > 4294967296.)
+    {
+      Ncur /= Dcur;
+      Nold /= Dcur;
+      Dold /= Dcur;
+      Dcur = 1.;
+    }
+    // normalize if necessary with 2^32
+    if (std::abs(Ncur) > 4294967296.)
+    {
+      Ncur /= 4294967296.;
+      Nold /= 4294967296.;
+      Dold /= 4294967296.;
+      Dcur /= 4294967296.;
+    }
+    // test D_n not too small
+    if (std::abs(Dcur) != 0)
+    {
+      Real cfold = cf;
+      cf = Ncur/Dcur;
+      // check cv
+      if (std::abs(cf - cfold) < std::abs(cf)*Arithmetic<Real>::epsilon()) { break;}
+    }
+  }
+  return lower_tail ? bt*cf/a : 1-bt*cf/a;
 }
 
 /** @ingroup Analysis
- *  @brief compute the odd coefs of the beta Ratio function
- *  serie expansion.
+ *  @brief compute the odd coefficients of the beta Ratio function
+ *  asymptotic expansion.
  *
- *  Given the 2t first coefs, compute and return the (2t+1)-th
+ *  Given the 2t first coefficients, compute and return the (2t+1)-th
  *  coefficient. The value is stored to the back of the Vector.
  *
- *  @param A vector of dimension 1:n, n>=2, of the coefs
- *  @param std the binomial standard deviation \f$ \sqrt{pq} \f$
- *  @param qmp the difference q-p
+ *  @param A vector of dimension 0:n, n=4,6,8,... of the coefficients
+ *  @return the (n+1)-th coefficient
  */
-static Real coefs_odd_se( Real const& std, Real const& qmp, std::vector<Real> &A)
+static Real coefs_odd_se(std::vector<Real> &A)
 {
-  // get the number of exising coefs (should be even)
-  int n = A.size();
-  int t = n/2;
-  // know case
-  if (t==1)
-  {
-    // compute result
-    Real res = (qmp/3+std)*(qmp/3-std)/(4*std);
-    // save it in A
-    A.push_back(res);
-    // and return it
-    return res;
-  }
+  // get the index of the current coefficient (should be n = 2 * l with l>=1)
+  int n = A.size()-1;
+  int l = n/2;
+
   // initialize the sums
-  Real res, sum1 =0, sum2 =0;
-  // update sum
-  for (int k=1, k1=0; k<t; k++, k1++)
+  Real U2lp1 =A[1]*A[n-1], T2lp1 =0;
+  for (int k=2; k<l; k++)
   {
-    sum1 += A[k] *A[n+k];
-    sum2 += A[k1]*A[n-k1];
+    U2lp1 += A[k] * A[n-k];
+    T2lp1 += A[k+1]*A[n+1-k];
   }
   // compute the result
-  res = ( ((qmp*A[n-1] - A[t-1]*A[t-1])/2 - sum2)/(t+1)
-          - sum1- A[t]*A[t]/2
-        )/std;
+  Real res = -( A[2]*A[n]*(n-1.)
+              + 2* U2lp1 + (n+2.) * T2lp1
+              + A[l]*A[l] + (l+1.) * A[l+1]*A[l+1]
+              ) / ((n+2.) * A[1]);
   // save it in A
   A.push_back(res);
   // and return it
@@ -517,169 +449,121 @@ static Real coefs_odd_se( Real const& std, Real const& qmp, std::vector<Real> &A
 }
 
 /** @ingroup Analysis
- *  @brief compute the even coefs of the beta Ratio function
- *  serie expansion.
+ *  @brief compute the even coefficients of the beta Ratio function
+ *  asymptotic expansion.
  *
- *  Given the 2t+1 first coefs, compute and return the (2t+2)-th
+ *  Given the n=2l-1 first coefficients, compute and return the (2l)-th
  *  coefficient. The value is stored to the back of the Vector.
  *
- *  @param A vector of dimension 1:n, n>=3, of the coefs
- *  @param std the binomial standard deviation \f$ \sqrt{pq} \f$
- *  @param qmp the difference q-p
+ *  @param A vector of dimension 0:n, n=5,7,... of the coefficients
+ *  @return the (n+1)-th coefficient
  */
-static Real coefs_even_se( Real const& std, Real const& qmp, std::vector<Real> &A)
+static Real coefs_even_se( std::vector<Real> &A)
 {
-  // get the number of exising coefs (should be odd)
-  int n = A.size();
-  int t = n/2;
-  // know case
-  if (t==1)
-  {
-    // compute result
-    Real res = -qmp*(1+A[2]/std)*2/15;
-    // save it in A
-    A.push_back(res);
-    // and return it
-    return res;
-  }
+  // get the current coefficient (should be n = 2 * l - 1 with l>=1)
+  int n = A.size()-1;
+  int l = (n+1)/2;
   // initialize the sums
-  Real res, sum1 =0, sum2 =0;
+  Real U2l =A[1]*A[n-1], T2l =0;
   // update sum
-  for (int k=1, k1=0; k<=t; k++, k1++)
+  for (int k=2; k<l; k++)
   {
-    sum1 += A[k]*A[n-k];
-    sum2 += A[k1] *A[n-k1];
+    U2l += A[k]*A[n-k];
+    T2l += A[k+1] *A[n+1-k];
   }
   // compute the result
-  res = ( (qmp*A[n-1] - 2*sum2)/(n+2) - sum1)/std;
+  Real res = -(A[2]*A[n]*(n-1.) + 2.*U2l + (n+2.) * T2l ) / ((n+2.) * A[1]);
   // save it in A
   A.push_back(res);
   // and return it
   return res;
 }
 
+/* @ingroup Analysis
+ *  @brief compute the coefficients of the beta Ratio function
+ *  asymptotic expansion.
+ *
+ *  Given the n=2l+1 first coefficients, compute and return the (2l+2)-th
+ *  coefficient. The value is stored to the back of the Vector.
+ *
+ *  @param A vector of dimension 0:n, n=5,7,... of the coefficients
+ *  @return the (n+1)-th coefficient
+ */
+//static Real coefs_se( std::vector<Real> &A)
+//{
+//  // get the number of existing coefficient
+//  int n = A.size();
+//  // sum
+//  Real Sn =2*A[1]*A[n-2];
+//  for (int k=2; k<=n-3; k++)
+//  { Sn += (k+1) * A[k+1]*A[n-k] + A[k] * A[n-1-k];}
+//  // compute the result
+//  Real res = -((n-2.)*A[2]*A[n-1]/A[1] + Sn/A[1]) / (n+1.);
+//  // save it in A
+//  A.push_back(res);
+//  // and return it
+//  return res;
+//}
+
 /** @ingroup Analysis
  *  @brief Compute the incomplete beta function ratio I_x(a,b)
- *  using its serie expansion.
+ *  using its series expansion.
  *
  * Compute the incomplete beta function ratio I_x(a,b)
- * using it's serie expansion.
- *  @param a first parameter, must be >0
- *  @param b second parameter, must be >0
+ * using it's series expansion.
+ *  @param a, b parameters, must be >0
  *  @param x value to evaluate the function
- *  @param xm1 @c true if x is in [q,p], @c false otherwise
+ *  @param xm1 @c true if we want 1-x value, @c false otherwise
  *  @param lower_tail @c true if we want the lower tail, @c false otherwise
- *  @param iterMax Maximal number of iteration (default is 20)
  **/
-static Real betaRatio_se( Real const& a
-                        , Real const& b
-                        , Real const& x
-                        , bool xm1
-                        , bool lower_tail
-                        , int const& iterMax = 20
-                        )
+Real betaRatio_se( Real const& a, Real const& b, Real const& x
+                 , bool xm1, bool lower_tail
+                 )
 {
+#ifdef STK_BETARATIO_DEBUG
+  stk_cout << _T("BetaRatio_se(") << a << _T(", ") << b << _T(", ") << x << _T(")\n");
+#endif
   // parameters
-  Real s  = a+b;
-  Real p  = a/s, q  = b/s;
-  Real sx = s*x;
-  Real sy = s-sx;
-  //Real D  = a*log(x/p)+b*log(y/q);
-  Real D  = (xm1 ? dev0(a, sy)+dev0(b, sx) : dev0(a, sx)+dev0(b, sy));
-
-  Real z2 = 2 * D;
-  Real z  = (((x<p)&&!xm1)||((x>q)&&xm1)) ? -sqrt(z2) : sqrt(z2);
-
+  Real s  = a+b, p  = a/s, q  = b/s, sx = s*x, sy = s-sx;
+  //Real z2  = 2 * (a+b) (p*log(x/p)+q*log(y/q));
+  Real z2 = 2. * (xm1 ? dev0(a, sy)+dev0(b, sx) : dev0(a, sx)+dev0(b, sy));
+  Real z  = (((x<p)&&!xm1)||((x>q)&&xm1)) ? -std::sqrt(z2) : std::sqrt(z2);
   //  Compute normal cdf
   Real pnorm = lower_tail ?  normal_cdf_raw( z) : normal_cdf_raw(-z);
   //  Compute normal pdf
-  Real dnorm = Const::_1_SQRT2PI_ * exp(-D);
+  Real dnorm = Const::_1_SQRT2PI_ * exp(-z2/2.);
   // check large values of z
-  if (!dnorm) return pnorm;
-  // epsilon value for the serie
-  Real eps = Arithmetic<Real>::epsilon()*pnorm;
+  if (dnorm < Arithmetic<Real>::epsilon()) return pnorm;
 
   // auxiliary variables
-  Real std = sqrt(a)*sqrt(b)/s, qmp = q - p;
+  Real a1 = std::sqrt(a)*std::sqrt(b)/s, a2 = (b - a)/(3.*s), a2_a1 = a2/a1, sqrts = std::sqrt(s);
 
-  // serie coefs
-  std::vector<Real> A(2);
-  A.reserve(iterMax); // reserve enough space
-  A[0] = std;
-  A[1] = qmp/3;
-  // variables for the series
-  Real odd_term  = 1;
-  Real even_term = 2/sqrt(s);
-
-  Real sum1_term = z, sum1 = z;
-  Real sum2_term = 1, sum2 = 1;
-
-  // variables for the numerator and the denominator
-  Real num = qmp*even_term/3; // = A[1]*even_term;
-  Real den = std;             // = A[0];
+  // series coefs
+  std::vector<Real> A, B;
+  A.reserve(10); // reserve enough space
+  A.push_back(p); // a0
+  A.push_back(a1); // a1
+  A.push_back(a2); // a2
+  A.push_back(a1 * (a2_a1/2.+0.5) * (a2_a1/2.-0.5)); //a3
+  A.push_back(-0.4*a2*((a2_a1/2.+0.5) * (a2_a1/2.-0.5)+1.)); // a4
+  // compute the numerator and the denominator
+  Real num = (a2 * 2.
+             + (A[3] * 3. * z
+               + (A[4] * 4. * (2+ z2)
+                 + (coefs_odd_se(A) * 5. * z * (3 + z2)
+                   + (coefs_even_se(A) * 6. * (2*4 + z2 * (4. + z2))
+                     + (coefs_odd_se(A) * 7. * z *(5*3 + z2*(5 + z2))
+                       + (coefs_even_se(A) * 8. * (2*4*6 + z2 * (4*6 + z2 *(6 + z2)))
+                         + (coefs_odd_se(A) * 9. * z * (7*5*3 + z2 * (7*5 + z2 * (7+z2)))
+                             + (coefs_even_se(A) * 10. * (2*4*6*8 + z2 * (4*6*8 + z2 *(6*8 + z2 * (8 + z2))))
+                               + (coefs_odd_se(A) * 11. * z * (9*7*5*3 + z2 * (9*7*5 + z2 * (9*7+z2 * (9 + z2))))
+                               )/sqrts)/sqrts)/sqrts)/sqrts)/sqrts)/sqrts)/sqrts)/sqrts)/sqrts)/sqrts;
+  Real den = a1 + (A[3] + (A[5] + (A[7] + (A[9] + A[11] * 11./s) * 9./s)*7./s)*5./s)*3./s;
 
   // compute ratio
-  Real res = num/den;
-
-  // computation
-  for (int k=1; k<iterMax; k++)
-  {
-    // odd_term : (2k+1)!!/(a+b)^k
-    odd_term *= (2*k+1)/s;
-    // even_term (2*k)!!/(a+b)^{k+1/2}
-    even_term *= 2*(k+1)/s;
-
-    // compute a_{2k+1}
-    Real a_n = coefs_odd_se(std, qmp, A);
-    // check result
-    if (Arithmetic<Real>::isFinite(a_n))
-    {
-      // denominator \sum_0^k (2j+1)!!a_{2j+1}/(a+b)^j
-      den += odd_term * a_n;
-      // num+=\sum_1^k (2k+1)!!a_{2k+1}/(a+b)^k (\sum_1^k z^{2j-1}/(2j-1)!!)
-      num += odd_term * a_n * sum1;
-
-      // compute a_{2k+2}
-      a_n = coefs_even_se(std, qmp, A);
-      // check result
-      if (Arithmetic<Real>::isFinite(a_n))
-      {
-        // update sum2_term = z^{2k}/(2k)!!
-        // update sum2      = \sum_0^k z^{2j}/(2j)!!
-        sum2 += (sum2_term *= z2/(2*k));
-        // num+=\sum_0^k (2k+2)!!a_{2k+2}/(a+b)^k (\sum_1^k z^{2j}/(2j)!!)
-        num += even_term * a_n * sum2;
-      }
-    }
-    // test cv
-    Real old_res = res;
-    res = num/den;
-    if (std::abs(old_res - res)*dnorm < eps)
-    { break;}
-
-    // update sum1_term = z^{2k+1}/(2k+1)!!
-    // update sum1      = \sum_1^{k+1} z^{2j-1}/(2j-1)!!
-    sum1 += (sum1_term *= z2/(2*k+1));
-  }
-#ifdef STK_DEBUG
+  Real ratio = num/den;
   // result
-  Real result = lower_tail ? pnorm - res * dnorm
-                           : pnorm + res * dnorm;
-  if (!Arithmetic<Real>::isFinite(result))
-  {
-    std::cout << "Error in betRatio_se\n"
-              << "pnorm = " << pnorm
-              << " res = " << res
-              << " dnorm  = " << dnorm
-              << "lt =  " << lower_tail
-              << _T("\n";);
-  }
-  return result;
-#else
-  // result
-    return lower_tail ? pnorm - res * dnorm
-                      : pnorm + res * dnorm;
-#endif
+  return lower_tail ? pnorm - ratio * dnorm : pnorm + ratio * dnorm;
 }
 
 /** @ingroup Analysis
@@ -694,57 +578,61 @@ static Real betaRatio_se( Real const& a
  *  @param x value to evaluate the function
  *  @param lower_tail @c true if we want the lower tail, @c false otherwise
  **/
-Real betaRatio( Real const& a, Real const& b, Real const& x
-              , bool lower_tail)
+Real betaRatio( Real const& a, Real const& b, Real const& x, bool lower_tail)
 {
   // Check if a and x are available
-  if (  Arithmetic<Real>::isNA(a)
-     || Arithmetic<Real>::isNA(b)
+  if (  Arithmetic<Real>::isNA(a) || Arithmetic<Real>::isNA(b)
      || Arithmetic<Real>::isNA(x)
      ) return Arithmetic<Real>::NA();
   // Negative parameter not allowed
   if ((a<=0)||(b<=0))
-    throw domain_error("Funct::betaRatio(a,b,x) "
-                            "Negative parameter a or b");
+    STKDOMAIN_ERROR_2ARG(Funct::betaRatio,a,b,negative parameter not allowed);
   // trivial case
   if (x<=0) return lower_tail ? 0. : 1.;
   if (x>=1) return lower_tail ? 1. : 0.;
-  // compute 1-x
-  Real y = (0.5 - x) + 0.5;
   // parameters
   Real p=a/(a+b);
-  // small a or b
+  // small a,b max(a,b) <= 15
+  if (std::max(a,b)<=15)
+  {
+    if ((a<=1 && x <=0.5)||(b<=1 && x>=0.5))
+    {
+      return (x <=0.5) ? betaRatio_sr(a, b, x, false, lower_tail)
+                       : betaRatio_sr(b, a, x, true, !lower_tail);
+    }
+   // (a<=1) and (x>0.5), use serie_up for b
+   if (a<=1) return betaRatio_up(b, a, x, true, !lower_tail);
+   // (b<=1) and (x<0.5), use serie_up for a
+   if (b<=1) return betaRatio_up(a, b, x, false, lower_tail);
+   // min(a,b)>1, use serie_up for a or b
+   return (x <=0.5) ? betaRatio_up(a, b, x, false, lower_tail)
+                    : betaRatio_up(b, a, x, true, !lower_tail);
+  }
+  // min(a,b) < 1 and max(a,b) > 15
   if (std::min(a,b)<=1)
   {
+//    std::cout << "case 1" << std::endl;
     // case b<=1 and a>=15
     if (a>=15) return betaRatio_ae(a, b, x, false, lower_tail);
     // case a<=1 and b>=15
     if (b>=15) return betaRatio_ae(b, a, x, true, !lower_tail);
-    // ((a<15) and (b<15) and (min(a,b)<=1))
-    return (x<=0.5) ? betaRatio_sr(a, b, x, lower_tail)
-                    : betaRatio_sr(b, a, y, !lower_tail);
   }
-  // (1<a<=80) and (1<b<=80)
-  if (std::max(a,b)<=120)
+
+  // ((1<a<=60)) or ((1<b<=60))
+  if ((std::min(a,b)<=60))
   {
-    // general case
-    return (x < p) ? betaRatio_cf(a, b, x,  lower_tail)
-                   : betaRatio_cf(b, a, y, !lower_tail);
+//      std::cout << "case 3.2" << std::endl;
+    return (a<b) ? betaRatio_up(a, b, x, false, lower_tail)
+                 : betaRatio_up(b, a, x, true, !lower_tail);
   }
-  // ((1<a<=15) and (120<b)) or ((120<a) and (1<b<=15))
-  if (std::min(a,b)<=15)
+  // general case
+  if ((x<0.97*p)||(x>1.03*p))
   {
-    return (a<=15) ? betaRatio_up(a, b, x, false, lower_tail)
-                   : betaRatio_up(b, a, x, true, !lower_tail);
+//      std::cout << "case 2.1" << std::endl;
+    return (x < p) ? betaRatio_cf(a, b, x, false, lower_tail)
+                   : betaRatio_cf(b, a, x, true, !lower_tail);
   }
-  // ()
-  if ((std::min(a,b)<=120)&& (std::max(a,b)<=200))
-  {
-    // general case
-    return (x < p) ? betaRatio_cf(a, b, x,  lower_tail)
-                   : betaRatio_cf(b, a, y, !lower_tail);
-  }
-  // other cases
+//      std::cout << "case 2.2" << std::endl;
   return (a<b) ? betaRatio_se(a-1, b-1, x, false, lower_tail)
                : betaRatio_se(b-1, a-1, x, true, !lower_tail);
 }
